export type CacheOptions = {
    /** Delete keys automatically after x amount of seconds */
    ttl?: number;
    /** Storage to persist cache */
    storage?: Storage;
    /** Key cache will be stored under */
    storageKey?: string;
};
/**
 * Map of data which tracks whether it is a complete collection & offers optional expiry of cached values
 */
export declare class Cache<K extends string | number | symbol, T> {
    readonly key?: keyof T | undefined;
    readonly options: CacheOptions;
    private store;
    /** Support index lookups */
    [key: string | number | symbol]: T | any;
    /** Whether cache is complete */
    complete: boolean;
    /**
     * Create new cache
     *
     * @param {keyof T} key Default property to use as primary key
     * @param options
     */
    constructor(key?: keyof T | undefined, options?: CacheOptions);
    private getKey;
    /**
     * Get all cached items
     *
     * @return {T[]} Array of items
     */
    all(): T[];
    /**
     * Add a new item to the cache. Like set, but finds key automatically
     *
     * @param {T} value Item to add to cache
     * @param {number | undefined} ttl Override default expiry
     * @return {this}
     */
    add(value: T, ttl?: any): this;
    /**
     * Add several rows to the cache
     *
     * @param {T[]} rows Several items that will be cached using the default key
     * @param complete Mark cache as complete & reliable, defaults to true
     * @return {this}
     */
    addAll(rows: T[], complete?: boolean): this;
    /**
     * Remove all keys from cache
     */
    clear(): void;
    /**
     * Delete an item from the cache
     *
     * @param {K} key Item's primary key
     */
    delete(key: K): void;
    /**
     * Return cache as an array of key-value pairs
     * @return {[K, T][]} Key-value pairs array
     */
    entries(): [K, T][];
    /**
     * Get item from the cache
     * @param {K} key Key to lookup
     * @return {T} Cached item
     */
    get(key: K): T;
    /**
     * Get a list of cached keys
     *
     * @return {K[]} Array of keys
     */
    keys(): K[];
    /**
     * Get map of cached items
     *
     * @return {Record<K, T>}
     */
    map(): Record<K, T>;
    /**
     * Add an item to the cache manually specifying the key
     *
     * @param {K} key Key item will be cached under
     * @param {T} value Item to cache
     * @param {number | undefined} ttl Override default expiry in seconds
     * @return {this}
     */
    set(key: K, value: T, ttl?: number | undefined): this;
    /**
     * Get all cached items
     *
     * @return {T[]} Array of items
     */
    values: T[];
}
